import React, { Component } from "react";
import axios from "axios";

import AppContext from "@context/AppContext";

import Scroll from "@utils/Scroll";

import endpoints from "@services/Api/endpoints";

import NoData from "@components/NoData";
import Table from "@components/Table";
import FrmUser from "@components/FrmUser";

export default class UsersTable extends Component {
    constructor() {
        super();

        this.state = {
            showFrmUser: false,
            users: [],
            user: null,
            filtered: [],
            stickyRows: [],
            scrollableRows: [],
            stickyColumns: [
                {
                    width: 100,
                    name: "ID",
                    filter: null,
                },
            ],
            scrollableColumns: [
                {
                    width: 300,
                    name: "Nombre",
                    filter: null,
                },
                {
                    width: 300,
                    name: "Correo",
                    filter: null,
                },
                {
                    width: 150,
                    name: "Rol",
                    filter: null,
                },
                {
                    width: 200,
                    name: "Fecha de alta",
                    filter: null,
                },
            ],
        };
    }

    static contextType = AppContext;

    getUserRow(user, scrollable = true) {
        return scrollable
            ? {
                  id: user.id,
                  rowClass: user.deleted ? "Deleted" : "",
                  cells: [
                      {
                          id: Math.random() * 100,
                          width: 300,
                          value: user.name,
                      },
                      {
                          id: Math.random() * 100,
                          width: 300,
                          value: user.email,
                      },
                      {
                          id: Math.random() * 100,
                          width: 150,
                          value: user.role.title,
                      },
                      {
                          id: Math.random() * 100,
                          width: 200,
                          value: user.registered_at,
                      },
                  ],
                  actions: [],
              }
            : {
                  id: user.id,
                  rowClass: user.deleted ? "Deleted" : "",
                  cells: [
                      {
                          id: Math.random() * 100,
                          width: 100,
                          value: user.id,
                          onClick: () =>
                              this.setState({
                                  user: user,
                                  showFrmUser: true,
                              }),
                          isEditor: true,
                      },
                  ],
              };
    }

    addUserRow(user) {
        this.setState({
            stickyRows: [
                this.getUserRow(user, false),
                ...this.state.stickyRows,
            ],
            scrollableRows: [
                this.getUserRow(user),
                ...this.state.scrollableRows,
            ],
        });
    }

    updateUserRow(user) {
        const updatedScrollableRows = this.state.scrollableRows.map(
            (oldRow) => {
                if (oldRow.id == user.id) {
                    return this.getUserRow(user);
                }

                return oldRow;
            }
        );

        const updatedStickyRows = this.state.stickyRows.map((oldRow) => {
            if (oldRow.id == user.id) {
                return this.getUserRow(user, false);
            }

            return oldRow;
        });

        this.setState({
            scrollableRows: updatedScrollableRows,
            stickyRows: updatedStickyRows,
        });
    }

    async fetchUsers() {
        await axios
            .get(endpoints.users.get)
            .then((response) => {
                const users = response.data.users;

                const scrollableRows = users.map((user) => {
                    return this.getUserRow(user);
                });

                const stickyRows = users.map((user) => {
                    return this.getUserRow(user, false);
                });

                this.setState({
                    stickyRows: stickyRows,
                    scrollableRows: scrollableRows,
                });
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    componentDidMount() {
        this.fetchUsers();
    }

    render() {
        return (
            <div className='UsersTable'>
                <section className='Page__Container'>
                    <div className='Page__Options'>
                        <div
                            className='Option'
                            onClick={() =>
                                this.setState({ showFrmUser: true })
                            }>
                            Nuevo
                        </div>
                    </div>
                    <div className='Page__Wrapper'>
                        {this.state.scrollableRows.length == 0 ? (
                            <NoData message='Sin registro de usuarios' />
                        ) : null}
                        <div className='Content'>
                            <div className='Sticky__Container'>
                                <Table
                                    onscroll={() =>
                                        Scroll.onscroll(
                                            ".UsersTable .Page__Wrapper .Content .Sticky__Container .Table .Body",
                                            ".UsersTable .Page__Wrapper .Content .Tbl__Scrollable .Table .Body"
                                        )
                                    }
                                    columns={this.state.stickyColumns}
                                    rows={this.state.stickyRows}
                                />
                            </div>
                            <div className='Tbl__Scrollable'>
                                <Table
                                    onscroll={() =>
                                        Scroll.onscroll(
                                            ".UsersTable .Page__Wrapper .Content .Tbl__Scrollable .Table .Body",
                                            ".UsersTable .Page__Wrapper .Content .Sticky__Container .Table .Body"
                                        )
                                    }
                                    columns={this.state.scrollableColumns}
                                    rows={this.state.scrollableRows}
                                />
                            </div>

                            {this.state.showFrmUser ? (
                                <FrmUser
                                    user={this.state.user}
                                    newUserRow={(user) => this.addUserRow(user)}
                                    updateUserRow={(user) =>
                                        this.updateUserRow(user)
                                    }
                                    closeModal={() =>
                                        this.setState({
                                            showFrmUser: false,
                                            user: null,
                                        })
                                    }
                                />
                            ) : null}
                        </div>
                    </div>
                </section>
            </div>
        );
    }
}
